@@ -40,14 +40,14 @@ class MinMaxObserver(ObserverBase):
         self.num_flag = 0
         self.out_channels = out_channels
         if self.q_level == 'L':
-            self.min_val = torch.zeros((1), dtype=torch.float32, device=self.device)
-            self.max_val = torch.zeros((1), dtype=torch.float32, device=self.device)
+            self.register_buffer('min_val', torch.zeros((1), dtype=torch.float32))
+            self.register_buffer('max_val', torch.zeros((1), dtype=torch.float32))
         elif self.q_level == 'C':
-            self.min_val = torch.zeros((out_channels, 1, 1, 1), dtype=torch.float32, device=self.device)
-            self.max_val = torch.zeros((out_channels, 1, 1, 1), dtype=torch.float32, device=self.device)
+            self.register_buffer('min_val', torch.zeros((out_channels, 1, 1, 1), dtype=torch.float32))
+            self.register_buffer('max_val', torch.zeros((out_channels, 1, 1, 1), dtype=torch.float32))
         elif self.q_level == 'FC':
-            self.min_val = torch.zeros((out_channels, 1), dtype=torch.float32, device=self.device)
-            self.max_val = torch.zeros((out_channels, 1), dtype=torch.float32, device=self.device)
+            self.register_buffer('min_val', torch.zeros((out_channels, 1), dtype=torch.float32))
+            self.register_buffer('max_val', torch.zeros((out_channels, 1), dtype=torch.float32))
 
     def update_range(self, min_val_cur, max_val_cur):
         if self.q_level == 'C':
@@ -70,14 +70,14 @@ class MovingAverageMinMaxObserver(ObserverBase):
         self.num_flag = 0
         self.out_channels = out_channels
         if self.q_level == 'L':
-            self.min_val = torch.zeros((1), dtype=torch.float32, device=self.device)
-            self.max_val = torch.zeros((1), dtype=torch.float32, device=self.device)
+            self.register_buffer('min_val', torch.zeros((1), dtype=torch.float32))
+            self.register_buffer('max_val', torch.zeros((1), dtype=torch.float32))
         elif self.q_level == 'C':
-            self.min_val = torch.zeros((out_channels, 1, 1, 1), dtype=torch.float32, device=self.device)
-            self.max_val = torch.zeros((out_channels, 1, 1, 1), dtype=torch.float32, device=self.device)
+            self.register_buffer('min_val', torch.zeros((out_channels, 1, 1, 1), dtype=torch.float32))
+            self.register_buffer('max_val', torch.zeros((out_channels, 1, 1, 1), dtype=torch.float32))
         elif self.q_level == 'FC':
-            self.min_val = torch.zeros((out_channels, 1), dtype=torch.float32, device=self.device)
-            self.max_val = torch.zeros((out_channels, 1), dtype=torch.float32, device=self.device)
+            self.register_buffer('min_val', torch.zeros((out_channels, 1), dtype=torch.float32))
+            self.register_buffer('max_val', torch.zeros((out_channels, 1), dtype=torch.float32))
 
     def update_range(self, min_val_cur, max_val_cur):
         if self.q_level == 'C':
@@ -101,8 +101,8 @@ class HistogramObserver(nn.Module):
         self.momentum = momentum
         self.percentile = percentile
         self.num_flag = 0
-        self.min_val = torch.zeros((1), dtype=torch.float32, device=self.device)
-        self.max_val = torch.zeros((1), dtype=torch.float32, device=self.device)
+        self.register_buffer('min_val', torch.zeros((1), dtype=torch.float32))
+        self.register_buffer('max_val', torch.zeros((1), dtype=torch.float32))
 
     @torch.no_grad()
     def forward(self, input):
@@ -119,17 +119,29 @@ class HistogramObserver(nn.Module):
 
 
 # ********************* quantizers（量化器，量化） *********************
-# 取整(ste)
+# 取整(饱和/截断ste)
 class Round(Function):
     @staticmethod
-    def forward(self, input):
+    def forward(self, input, observer_min_val, observer_max_val, q_type):
+        # 对称
+        if q_type == 0:
+            max_val = torch.max(torch.abs(observer_min_val), torch.abs(observer_max_val))
+            min_val = -max_val
+        # 非对称
+        else:
+            max_val = observer_max_val
+            min_val = observer_min_val
+        self.save_for_backward(input, min_val, max_val)
         output = torch.round(input)
         return output
 
     @staticmethod
     def backward(self, grad_output):
+        input, min_val, max_val= self.saved_tensors
         grad_input = grad_output.clone()
-        return grad_input
+        grad_input[input.gt(max_val)] = 0
+        grad_input[input.lt(min_val)] = 0
+        return grad_input, None, None, None
 
 class Quantizer(nn.Module):
     def __init__(self, bits, observer, activation_weight_flag, qaft=False):
@@ -138,6 +150,7 @@ class Quantizer(nn.Module):
         self.observer = observer
         self.activation_weight_flag = activation_weight_flag
         self.qaft = qaft
+        self.q_type = 0
         # scale/zero_point/eps
         if self.observer.q_level == 'L':
             self.register_buffer('scale', torch.ones((1), dtype=torch.float32))
@@ -152,12 +165,12 @@ class Quantizer(nn.Module):
 
     def update_qparams(self):
         raise NotImplementedError
-
+    
     # 取整(ste)
-    def round(self, input):
-        output = Round.apply(input)
+    def round(self, input, observer_min_val, observer_max_val, q_type):
+        output = Round.apply(input, observer_min_val, observer_max_val, q_type)
         return output
-
+    
     def forward(self, input):
         if self.bits == 32:
             output = input
@@ -168,11 +181,12 @@ class Quantizer(nn.Module):
             if not self.qaft:
                 #qat, update quant_para
                 if self.training:
-                    self.observer(input)
+                    self.observer(input)   # update observer_min and observer_max
                     self.update_qparams()  # update scale and zero_point
-            # 量化/反量化
-            output = (torch.clamp(self.round(input / self.scale - self.zero_point),
-                                  self.quant_min_val, self.quant_max_val) + self.zero_point) * self.scale
+            output = (torch.clamp(self.round(input / self.scale - self.zero_point,
+                      self.observer.min_val / self.scale - self.zero_point,
+                      self.observer.max_val / self.scale - self.zero_point, self.q_type),
+                      self.quant_min_val, self.quant_max_val) + self.zero_point) * self.scale
         return output
 
 class SignedQuantizer(Quantizer):
@@ -202,6 +216,7 @@ class UnsignedQuantizer(Quantizer):
 # 对称量化
 class SymmetricQuantizer(SignedQuantizer):
     def update_qparams(self):
+        self.q_type = 0
         quant_range = float(self.quant_max_val - self.quant_min_val) / 2                                # quantized_range
         float_range = torch.max(torch.abs(self.observer.min_val), torch.abs(self.observer.max_val))     # float_range
         self.scale = float_range / quant_range                                                          # scale
@@ -211,6 +226,7 @@ class SymmetricQuantizer(SignedQuantizer):
 # 非对称量化
 class AsymmetricQuantizer(UnsignedQuantizer):
     def update_qparams(self):
+        self.q_type = 1
         quant_range = float(self.quant_max_val - self.quant_min_val)           # quantized_range
         float_range = self.observer.max_val - self.observer.min_val            # float_range
         self.scale = float_range / quant_range                                 # scale
@@ -400,12 +416,14 @@ class QuantBNFuseConv2d(QuantConv2d):
                  pretrained_model=False,
                  qaft=False,
                  ptq=False,
-                 percentile=0.9999):
+                 percentile=0.9999,
+                 bn_fuse_calib=False):
         super(QuantBNFuseConv2d, self).__init__(in_channels, out_channels, kernel_size, stride, padding, dilation, groups,
                                                 bias, padding_mode)
         self.num_flag = 0
         self.pretrained_model = pretrained_model
         self.qaft = qaft
+        self.bn_fuse_calib = bn_fuse_calib
         self.eps = eps
         self.momentum = momentum
         self.gamma = Parameter(torch.Tensor(out_channels))
@@ -495,15 +513,19 @@ class QuantBNFuseConv2d(QuantConv2d):
                         running_var = (1 - self.momentum) * self.running_var + self.momentum * batch_var
                         self.running_mean.copy_(running_mean)
                         self.running_var.copy_(running_var)
-                # BN融合
+                # bn融合
                 if self.bias is not None:
                     bias_fused = reshape_to_bias(self.beta + (self.bias - batch_mean) * (self.gamma / torch.sqrt(batch_var + self.eps)))
                 else:
                     bias_fused = reshape_to_bias(self.beta - batch_mean * (self.gamma / torch.sqrt(batch_var + self.eps)))  # b融batch
-                weight_fused = self.weight * reshape_to_weight(self.gamma / torch.sqrt(self.running_var + self.eps))        # w融running
+                  # bn融合不校准
+                if not self.bn_fuse_calib:    
+                    weight_fused = self.weight * reshape_to_weight(self.gamma / torch.sqrt(batch_var + self.eps))           # w融batch
+                  # bn融合校准
+                else:
+                    weight_fused = self.weight * reshape_to_weight(self.gamma / torch.sqrt(self.running_var + self.eps))    # w融running
             # 测试态
             else:
-                # BN融合
                 if self.bias is not None:
                     bias_fused = reshape_to_bias(self.beta + (self.bias - self.running_mean) * (self.gamma / torch.sqrt(self.running_var + self.eps)))
                 else:
@@ -511,7 +533,6 @@ class QuantBNFuseConv2d(QuantConv2d):
                 weight_fused = self.weight * reshape_to_weight(self.gamma / torch.sqrt(self.running_var + self.eps))                      # w融running
         else:
             #qaft, freeze bn_statis_para
-            # BN融合
             if self.bias is not None:
                 bias_fused = reshape_to_bias(self.beta + (self.bias - self.running_mean) * (self.gamma / torch.sqrt(self.running_var + self.eps)))
             else:
@@ -526,18 +547,24 @@ class QuantBNFuseConv2d(QuantConv2d):
             #qat, quant_bn_fuse_conv
             # 量化卷积
             if self.training:  # 训练态
-                output = F.conv2d(quant_input, quant_weight, None, self.stride, self.padding, self.dilation,
-                                self.groups)  # 注意，这里不加bias（self.bias为None）
-                # （这里将训练态下，卷积中w融合running参数的效果转为融合batch参数的效果）running ——> batch
-                output *= reshape_to_activation(torch.sqrt(self.running_var + self.eps) / torch.sqrt(batch_var + self.eps))
-                output += reshape_to_activation(bias_fused)
+                # bn融合不校准
+                if not self.bn_fuse_calib:
+                    output = F.conv2d(quant_input, quant_weight, bias_fused, self.stride, self.padding, self.dilation,
+                                      self.groups)
+                # bn融合校准
+                else:
+                    output = F.conv2d(quant_input, quant_weight, None, self.stride, self.padding, self.dilation,
+                                      self.groups)  # 注意，这里不加bias（self.bias为None）
+                    # （这里将训练态下，卷积中w融合running参数的效果转为融合batch参数的效果）running ——> batch
+                    output *= reshape_to_activation(torch.sqrt(self.running_var + self.eps) / torch.sqrt(batch_var + self.eps))
+                    output += reshape_to_activation(bias_fused)
             else:  # 测试态
                 output = F.conv2d(quant_input, quant_weight, bias_fused, self.stride, self.padding, self.dilation,
-                                self.groups)  # 注意，这里加bias，做完整的conv+bn
+                                  self.groups)  # 注意，这里加bias，做完整的conv+bn
         else:
             #qaft, quant_bn_fuse_conv
             output = F.conv2d(quant_input, quant_weight, bias_fused, self.stride, self.padding, self.dilation,
-                          self.groups)
+                              self.groups)
         return output
 
 
@@ -728,8 +755,8 @@ class QuantAdaptiveAvgPool2d(nn.AdaptiveAvgPool2d):
 
 
 def add_quant_op(module, a_bits=8, w_bits=8, q_type=0, q_level=0, device='cpu',
-                 weight_observer=0, bn_fuse=0, quant_inference=False, pretrained_model=False,
-                 qaft=False, ptq=False, percentile=0.9999):
+                 weight_observer=0, bn_fuse=False, bn_fuse_calib=False, quant_inference=False,
+                 pretrained_model=False, qaft=False, ptq=False, percentile=0.9999):
     for name, child in module.named_children():
         if isinstance(child, nn.Conv2d):
             if bn_fuse:
@@ -778,7 +805,8 @@ def add_quant_op(module, a_bits=8, w_bits=8, q_type=0, q_level=0, device='cpu',
                                                            pretrained_model=pretrained_model,
                                                            qaft=qaft,
                                                            ptq=ptq,
-                                                           percentile=percentile)
+                                                           percentile=percentile,
+                                                           bn_fuse_calib=bn_fuse_calib)
                     quant_bn_fuse_conv.bias.data = conv_child_temp.bias
                 else:
                     quant_bn_fuse_conv = QuantBNFuseConv2d(conv_child_temp.in_channels,
@@ -801,7 +829,8 @@ def add_quant_op(module, a_bits=8, w_bits=8, q_type=0, q_level=0, device='cpu',
                                                            pretrained_model=pretrained_model,
                                                            qaft=qaft,
                                                            ptq=ptq,
-                                                           percentile=percentile)
+                                                           percentile=percentile,
+                                                           bn_fuse_calib=bn_fuse_calib)
                 quant_bn_fuse_conv.weight.data = conv_child_temp.weight
                 quant_bn_fuse_conv.gamma.data = child.weight
                 quant_bn_fuse_conv.beta.data = child.bias
@@ -917,18 +946,21 @@ def add_quant_op(module, a_bits=8, w_bits=8, q_type=0, q_level=0, device='cpu',
         else:
             add_quant_op(child, a_bits=a_bits, w_bits=w_bits, q_type=q_type, q_level=q_level,
                          device=device, weight_observer=weight_observer, bn_fuse=bn_fuse,
-                         quant_inference=quant_inference, pretrained_model=pretrained_model,
+                         bn_fuse_calib=bn_fuse_calib, quant_inference=quant_inference,
+                         pretrained_model=pretrained_model,
                          qaft=qaft, ptq=ptq, percentile=percentile)
 
 
 def prepare(model, inplace=False, a_bits=8, w_bits=8, q_type=0, q_level=0,
-            device='cpu', weight_observer=0, bn_fuse=0, quant_inference=False,
-            pretrained_model=False, qaft=False, ptq=False, percentile=0.9999):
+            device='cpu', weight_observer=0, bn_fuse=False, bn_fuse_calib=False,
+            quant_inference=False, pretrained_model=False, qaft=False,
+            ptq=False, percentile=0.9999):
     if not inplace:
         model = copy.deepcopy(model)
     add_quant_op(model, a_bits=a_bits, w_bits=w_bits, q_type=q_type, q_level=q_level,
                  device=device, weight_observer=weight_observer, bn_fuse=bn_fuse,
-                 quant_inference=quant_inference, pretrained_model=pretrained_model,
+                 bn_fuse_calib=bn_fuse_calib, quant_inference=quant_inference,
+                 pretrained_model=pretrained_model,
                  qaft=qaft, ptq=ptq, percentile=percentile)
     return model
 
